@@ -9,11 +9,11 @@ import Protocol._
 /** An interface to an R interpreter.
 *
 * An object `R` is the instance of this class available in a Scala interpreter created by calling the function
-* `scalaInterpreter` from the package [[http://cran.r-project.org/package=rscala rscala]].  It is through this instance `R` that
+* `scala` from the package [[http://cran.r-project.org/package=rscala rscala]].  It is through this instance `R` that
 * callbacks to the original [[http://www.r-project.org R]] interpreter are possible.
 * 
 * In a JVM-based application, an instance of this class is created using its companion object.  See below.  The paths of the
-* rscala's JARs, for both Scala 2.10 and 2.11, are available from [[http://www.r-project.org R]] using `rscala::rscalaJar()`.
+* rscala's JARs (for all supported versions of Scala) are available from [[http://www.r-project.org R]] using `rscala::rscalaJar()`.
 * To get just the JAR for Scala 2.11, for example, use `rscala::rscalaJar("2.11")`.
 * 
 * {{{
@@ -37,30 +37,11 @@ import Protocol._
 *
 * @author David B. Dahl
 */
-class RClient private (private val scalaServer: ScalaServer, private val in: DataInputStream, private val out: DataOutputStream, private val debugger: Debugger, private val serializeOutputState: State) extends Dynamic {
+class RClient private (private val scalaServer: ScalaServer, private val in: DataInputStream, private val out: DataOutputStream, private val debugger: Debugger, private val serializeOutput: Boolean) extends Dynamic {
 
   /** __For rscala developers only__: Returns `TRUE` if debugging output is enabled. */
   def debug = debugger.value
 
-  /** __For rscala developers only__: Sets whether debugging output should be displayed. */
-  def debug_=(v: Boolean) = {
-    if ( v != debug ) {
-      if ( debug ) debugger.msg("Sending DEBUG request.")
-      out.writeInt(DEBUG)
-      out.writeInt(if ( v ) 1 else 0)
-      out.flush()
-      debugger.value = v
-    }
-  }
-
-  def serializeOutput = serializeOutputState.value
-
-  def serializeOutput_=(v: Boolean) = {
-    if ( scalaServer == null ) {
-      serializeOutputState.value = v
-    } else throw new IllegalStateException(s"Change this value in R via intpSettings(interpreter,serialize=${if (v) "TRUE" else "FALSE"})")
-  }
-
   /** Closes the interface to the R interpreter.
   * 
   * Subsequent calls to the other methods will fail.
@@ -618,7 +599,7 @@ class RClient private (private val scalaServer: ScalaServer, private val in: Dat
  * application.
 *
 * An object `R` is an [[RClient]] instance available in a Scala interpreter created by calling the function
-* `scalaInterpreter` from the package [[http://cran.r-project.org/package=rscala rscala]].  It is through this instance
+* `scala` from the package [[http://cran.r-project.org/package=rscala rscala]].  It is through this instance
 * `R` that callbacks to the original [[http://www.r-project.org R]] interpreter are possible.
 * 
 * The paths of the rscala's JARs, for both Scala 2.10 and 2.11, are available from [[http://www.r-project.org R]] using
@@ -639,8 +620,8 @@ object RClient {
   }
 
   private val defaultArguments = OS match {
-    case "windows" =>  Array[String]("--vanilla","--silent","--slave") 
-    case _ =>          Array[String]("--vanilla","--silent","--slave")
+    case "windows" =>  Array[String]("--no-save","--no-restore","--silent","--slave") 
+    case _ =>          Array[String]("--no-save","--no-restore","--silent","--slave")
   }
 
   private val interactiveArguments = OS match {
@@ -653,22 +634,53 @@ object RClient {
     case _ =>          """R"""
   }
 
-  private def findROnWindows: String = {
-    val NEWLINE = sys.props("line.separator")  
-    var result : String = null
-    for ( root <- List("HKEY_LOCAL_MACHINE","HKEY_CURRENT_USER") ) {
-      val out = new StringBuilder()
-      val logger = ProcessLogger((o: String) => { out.append(o); out.append(NEWLINE) },(e: String) => {})
+  def findROnWindows: String = {
+    def checkValidPath(path: String) = {
+      val file = new java.io.File(path + """\bin\R.exe""")
+      if ( file.exists && file.isFile ) file.getAbsolutePath
+      else ""
+    }
+    def versionCompare(a: String, b: String): Boolean = { 
+      val Seq(aa,bb) = Seq(a,b).map(_.takeWhile(_!='.'))
+      val Seq(al,bl) = Seq(a,b).map(_.length)
+      val Seq(aaa,bbb) = Seq(aa,bb).map(_.toInt)
+      val Seq(aal,bbl) = Seq(aa,bb).map(_.length)
+      if ( aaa == bbb ) { 
+        if ( ( al == aal ) && ( bl == bbl ) ) false
+        else if ( ( al == aal ) && ( bl != bbl ) ) true
+        else if ( ( al != aal ) && ( bl == bbl ) ) false
+        else versionCompare(a.substring(aal+1),b.substring(bbl+1))
+      }
+      else aaa < bbb 
+    }
+    def getPath(key: String): Option[String] = {
+      val cmd = "reg query " + key
       try {
-        ("reg query \"" + root + "\\Software\\R-core\\R\" /v \"InstallPath\"") ! logger
-        val a = out.toString.split(NEWLINE).filter(_.matches("""^\s*InstallPath\s*.*"""))(0)
-        result = a.split("REG_SZ")(1).trim() + """\bin\R.exe"""
+        val result1 = cmd.!!.trim.split("\\r\\n").map(_.trim)
+        val result2 = result1.filter(_.matches("^\\s*InstallPath.*"))
+        if ( result2.length == 1 ) {
+          val result3 = checkValidPath(result2(0).split("REG_SZ")(1).trim)
+          if ( result3 != "" ) Some(result3)
+          else None
+        } else {
+          val dir = result1.length match {
+            case 0 => ""
+            case 1 => result1.head
+            case _ => result1.zip(result1.map(x => new File(x).getName)).sortWith( (x,y) => {
+                versionCompare(x._2,y._2)
+              }).last._1
+          }
+          getPath(dir)
+        }
       } catch {
-        case _ : Throwable =>
+        case _: Throwable => None
       }
     }
-    if ( result == null ) throw new RuntimeException("Cannot locate R using Windows registry.")
-    else return result
+    for ( root <- List("HKEY_LOCAL_MACHINE","HKEY_CURRENT_USER") ) {
+      val result = getPath(root+"""\SOFTWARE\R-core\R""")
+      if ( result.isDefined ) return result.get
+    }
+    throw new RuntimeException("Cannot locate R using Windows registry.  Please explicitly specify its path.")
   }
 
   private def reader(debugger: Debugger, label: String)(input: InputStream) = {
@@ -686,15 +698,21 @@ object RClient {
   */
   def apply(): RClient = apply(defaultRCmd)
 
-  /** Returns an instance of the [[RClient]] class using the path specified by `rCmd` and specifying whether debugging
-  * output should be display and the `timeout` to establish a connection with the R interpreter.
+  /**
+  * Returns an instance of the [[RClient]] class by running the `R` executable in the path, specifying whether output
+  * should be serialized.
   */
-  def apply(rCmd: String, debug: Boolean = false, serializeOutput: Boolean = false, timeout: Int = 60): RClient = {
+  def apply(serializeOutput: Boolean): RClient = apply(defaultRCmd,serializeOutput)
+
+  /** Returns an instance of the [[RClient]] class, using the path specified by `rCmd` and specifying whether output
+  * should be serialized, whether debugging output should be displayed, and the `timeout` to establish a connection
+  * with the R interpreter.
+  */
+  def apply(rCmd: String, serializeOutput: Boolean = false, debug: Boolean = false, timeout: Int = 60): RClient = {
     var cmd: PrintWriter = null
     val command = rCmd +: ( defaultArguments ++ interactiveArguments )
     val processCmd = Process(command)
-    val serializeOutputState = new State(serializeOutput)
-    val debugger = new Debugger(System.out,"Scala",false,debug)
+    val debugger = new Debugger(debug,new PrintWriter(System.out),"Scala",false)
     val processIO = new ProcessIO(
       o => { cmd = new PrintWriter(o) },
       reader(debugger,""),
@@ -702,7 +720,7 @@ object RClient {
       true
     )
     val processInstance = processCmd.run(processIO)
-    val codeInR = Seq("common.R","globals.R","protocol.R","rServer.R","scalaInterpreter.R","zzz.R").map(resource => {
+    val codeInR = Seq("common.R","globals.R","protocol.R","rServer.R","scala.R","zzz.R").map(resource => {
       scala.io.Source.fromInputStream(getClass.getResourceAsStream("/R/"+resource)).getLines.mkString("\n")
     }).mkString("\n\n")
     val allCodeInR = s"""
@@ -729,11 +747,11 @@ object RClient {
     val sockets = new ScalaSockets(portsFile.getAbsolutePath,debugger)
     sockets.out.writeInt(OK)
     sockets.out.flush()
-    apply(null,sockets.in,sockets.out,debugger,serializeOutputState)
+    apply(null,sockets.in,sockets.out,debugger,serializeOutput)
   }
 
   /** __For rscala developers only__: Returns an instance of the [[RClient]] class.  */
-  def apply(scalaServer: ScalaServer, in: DataInputStream, out: DataOutputStream, debugger: Debugger, serializeOutputState: State): RClient = new RClient(scalaServer,in,out,debugger,serializeOutputState)
+  def apply(scalaServer: ScalaServer, in: DataInputStream, out: DataOutputStream, debugger: Debugger, serializeOutput: Boolean): RClient = new RClient(scalaServer,in,out,debugger,serializeOutput)
 
 }
 
